use serde::{Deserialize, Serialize};
use specs::prelude::*;
use specs::saveload::{UuidMarker, UuidMarkerAllocator};
use specs::Component;

use crate::logging::TuiLogState;

pub mod menu;

pub struct State {
    pub input: String,
    pub menu_status: menu::Status,
    pub log_state: TuiLogState,

    pub world: World,
}

impl State {
    pub fn new(log_state: TuiLogState) -> Self {
        let mut world = World::new();
        world.register::<UuidMarker>();
        world.register::<TagId>();
        world.register::<Location>();
        world.register::<Metadata>();

        world.insert(UuidMarkerAllocator::new());

        Self {
            input: String::with_capacity(64),
            log_state,
            menu_status: menu::Status::Main,

            world,
        }
    }
}

#[derive(
    Component, Serialize, Deserialize, Hash, PartialEq, Eq, Clone, Debug,
)]
pub struct TagId(pub String);

#[derive(Component, Serialize, Deserialize, PartialEq, Clone, Debug)]
#[serde(tag = "type", content = "data", rename_all = "SCREAMING_SNAKE_CASE")]
pub enum Location {
    InStorage,
    RentedTo { renter: String },
}

#[derive(Component, Serialize, Deserialize, PartialEq, Clone, Debug)]
pub struct Metadata {
    pub name: String,
}
